/*
 * HTTP 1.0 library implementation
 * File name: http_10.c
 * Author: Jaime Prez Crespo <japecre@lsub.org>
 * Last modification: June 22, 2004
 *
 * Copyright (c) 2004, Jaime Prez Crespo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * - Neither the name of the Rey Juan Carlos University nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "http_10.h"
#include "http_config.h"
#include "statistics.h"

int http_10_request(char *str, const int method, const char *uri, const int n)
{
    char *tmp = NULL;
    int bytes = 0, linelen;

    switch (method) {
    case GET:
        linelen = strlen("GET  HTTP/1.0\r\n") + strlen(uri) + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"GET %s HTTP/1.0\r\n",uri);
        break;
    case HEAD:
        linelen = strlen("HEAD  HTTP/1.0\r\n") + strlen(uri) + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HEAD %s HTTP/1.0\r\n",uri);
        break;
    case POST:
        linelen = strlen("POST  HTTP/1.0\r\n") + strlen(uri) + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"POST %s HTTP/1.0\r\n",uri);
        break;
    default:
        linelen = strlen("UNKNOWN  HTTP/1.0\r\n") + strlen(uri) + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"UNKNOWN %s HTTP/1.0\r\n",uri);
    }

    assert(tmp != NULL);
    strncat(str,tmp,(n < linelen) ? n : linelen);
    free(tmp);
    return bytes;
}

int http_10_status(char *str, const int status, const int n)
{
    char *tmp = NULL;
    int bytes = 0, linelen;

    assert(str != NULL);

    switch (status) {
    case OK:
        linelen = strlen("HTTP/1.0 200 OK\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 200 OK\r\n");
        break;
    case CREATED:
        linelen = strlen("HTTP/1.0 201 Created\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 201 Created\r\n");
        break;
    case ACCEPTED:
        linelen = strlen("HTTP/1.0 202 Accepted\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 202 Accepted\r\n");
        break;
    case NOCONTENT:
        linelen = strlen("HTTP/1.0 204 No Content\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 204 No Content\r\n");
        break;
    case MULCHOICES:
        linelen = strlen("HTTP/1.0 300 Multiple Choices\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 300 Multiple Choices\r\n");
        break;
    case MOVEDPERM:
        linelen = strlen("HTTP/1.0 301 Moved Permanently\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 301 Moved Permanently\r\n");
        break;
    /*
     * Google responses include HTTP/1.0 302 Found status.
     * To follow strictly HTTP/1.0 RFC, please uncomment the following code,
     * and comment the next FOUND case.
     */
/*    case MOVEDTEMP:
        linelen = strlen("HTTP/1.0 302 Moved Temporarily\r\n") + 1;
        tmp = malloc(linelen*sizeof(char));
        bytes = snprintf(tmp,linelen,"HTTP/1.0 302 Moved Temporarily\r\n");
        break;*/
    case FOUND:
        linelen = strlen("HTTP/1.0 302 Found\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 302 Found\r\n");
        break;
    case NOTMODIFIED:
        linelen = strlen("HTTP/1.0 304 Not Modified\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 304 Not Modified\r\n");
        break;    
    case BADREQUEST:
        linelen = strlen("HTTP/1.0 400 Bad Request\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 400 Bad Request\r\n");
        break;
    case UNAUTHORIZED:
        linelen = strlen("HTTP/1.0 401 Unauthorized\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 401 Unauthorized\r\n");
        break;
    case FORBIDDEN:
        linelen = strlen("HTTP/1.0 403 Forbidden\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 403 Forbidden\r\n");
        break;
    case NOTFOUND:
        linelen = strlen("HTTP/1.0 404 Not Found\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 404 Not Found\r\n");
        break;
    case NOTIMPLEM:
        linelen = strlen("HTTP/1.0 501 Not Implemented\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 501 Not Implemented\r\n");
        break;
    case INTERROR:
        linelen = strlen("HTTP/1.0 500 Internal Server Error\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 500 Internal Server Error\r\n");
        break;
    case BADGATEWAY:
        linelen = strlen("HTTP/1.0 502 Bad Gateway\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 502 Bad Gateway\r\n");
        break;
    case SERVICEUNA:
        linelen = strlen("HTTP/1.0 503 Service Unavailable\r\n") + 1;
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"HTTP/1.0 503 Service Unavailable\r\n");
        break;
    }

    assert(tmp != NULL);
    strncat(str,tmp,(n < linelen) ? n : linelen);
    free(tmp);
    return bytes;
}

int http_10_header(char *str, const int header, const int n, const char *value)
{
    int linelen, bytes = 0;
    char *tmp;
    
    assert(str != NULL);

    linelen = 3;
    if (value != NULL)
        linelen += strlen(value);    /* include trailing \r\n\0 */

    switch (header) {
    case CACHECONTROL:
        linelen += strlen("Cache-Control: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Cache-Control: %s\r\n",value);
        break;
    case CONNECTION:
        linelen += strlen("Connection: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Connection: %s\r\n",value);
        break;
    case DATE:
        linelen += strlen("Date: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Date: %s\r\n",value);
        break;
    case PRAGMA:
        linelen += strlen("Pragma: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Pragma: %s\r\n",value);
        break;
    case TRAILER:
        linelen += strlen("Trailer: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Trailer: %s\r\n",value);
        break;
    case TRANSFERENC:
        linelen += strlen("Transfer-Encoding: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Transfer-Encoding: %s\r\n",value);
        break;
    case UPGRADE:
        linelen += strlen("Upgrade: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Upgrade: %s\r\n",value);
        break;
    case VIA:
        linelen += strlen("Via: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Via: %s\r\n",value);
        break;
    case WARNING:
        linelen += strlen("Warning: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Warning: %s\r\n",value);
        break;
    case ACCEPT:
        linelen += strlen("Accept: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Accept: %s\r\n",value);
        break;
    case ACCEPTCHARSET:
        linelen += strlen("Accept-Charset: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Accept-Charset: %s\r\n",value);
        break;
    case ACCEPTENC:
        linelen += strlen("Accept-Encoding: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Accept-Encoding: %s\r\n",value);
        break;
    case ACCEPTLANG:
        linelen += strlen("Accept-Language: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Accept-Language: %s\r\n",value);
        break;
    case AUTHORIZATION:
        linelen += strlen("Authorization: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Authorization: %s\r\n",value);
        break;
    case EXPECT:
        linelen += strlen("Expect: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Expect: %s\r\n",value);
        break;
    case FROM:
        linelen += strlen("From: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"From: %s\r\n",value);
        break;
    case HOST:
        linelen += strlen("Host: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Host: %s\r\n",value);
        break;
    case IFMATCH:
        linelen += strlen("If-Match: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"If-Match: %s\r\n",value);
        break;
    case IFMODIFSINCE:
        linelen += strlen("If-Modified-Since: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"If-Modified-Since: %s\r\n",value);
        break;
    case IFNONEMATCH:
        linelen += strlen("If-None-Match: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"If-None-Match: %s\r\n",value);
        break;
    case IFRANGE:
        linelen += strlen("If-Range: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"If-Range: %s\r\n",value);
        break;
    case IFUNMODSINCE:
        linelen += strlen("If-Unmodified-Since: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"If-Unmodified-Since: %s\r\n",value);
        break;
    case MAXFORWARDS:
        linelen += strlen("Max-Forwards: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Max-Forwards: %s\r\n",value);
        break;
    case PROXYAUTHOR:
        linelen += strlen("Proxy-Author: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Proxy-Author: %s\r\n",value);
        break;
    case RANGE:
        linelen += strlen("Range: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Range: %s\r\n",value);
        break;
    case REFERER:
        linelen += strlen("Referer: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Referer: %s\r\n",value);
        break;
    case TE:
        linelen += strlen("TE: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"TE: %s\r\n",value);
        break;
    case USERAGENT:
        linelen += strlen("User-Agent: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"User-Agent: %s\r\n",value);
        break;
    case ALLOW:
        linelen += strlen("Allow: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Allow: %s\r\n",value);
        break;
    case CONTENTENC:
        linelen += strlen("Content-Encoding: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Encoding: %s\r\n",value);
        break;
    case CONTENTLANG:
        linelen += strlen("Content-Language: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Language: %s\r\n",value);
        break;
    case CONTENTLEN:
        linelen += strlen("Content-Length: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Length: %s\r\n",value);
        break;
    case CONTENTLOC:
        linelen += strlen("Content-Location: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Location: %s\r\n",value);
        break;
    case CONTENTMD5:
        linelen += strlen("Content-MD5: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-MD5: %s\r\n",value);
        break;
    case CONTENTRANGE:
        linelen += strlen("Content-Range: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Range: %s\r\n",value);
        break;
    case CONTENTTYPE:
        linelen += strlen("Content-Type: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Content-Type: %s\r\n",value);
        break;
    case EXPIRES:
        linelen += strlen("Expires: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Expires: %s\r\n",value);
        break;
    case LASTMODIFIED:
        linelen += strlen("Last-Modified: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Last-Modified: %s\r\n",value);
        break;
    case LOCATION:
        linelen += strlen("Location: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Location: %s\r\n",value);
        break;
    case SERVER:
        linelen += strlen("Server: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Server: %s\r\n",value);
        break;
    case WWWAUTH:
        linelen += strlen("WWW-Authenticate: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"WWW-Authenticate: %s\r\n",value);
        break;
    case PROXYCONN:
        linelen += strlen("Proxy-Connection: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Proxy-Connection: %s\r\n",value);
        break;
    case KEEPALIVE:
        linelen += strlen("Keep-Alive: ");
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,linelen,"Keep-Alive: %s\r\n",value);
        break;
    default:
        if ((tmp = malloc(linelen*sizeof(char))) == NULL)
            return ENOTENMEM;
        bytes = snprintf(tmp,strlen(value),"%s\r\n",value);
        break;
    }

    assert(tmp != NULL);
    strncat(str,tmp,(n < linelen) ? n : linelen);

    free(tmp);
    return bytes;
}

ssize_t http_10_errmsg(const int fd,
                       const int code,
                       const char *reason,
                       const int headers,
                       const int type)
{
    int n, file;
    ssize_t bytes = 0;
    char resp[MAXBUFSIZE] = "";
    char reasonlen[10] = "";
    struct stat *errstats;
    char *filename;
    
    if (http_10_status(resp,code,MAXBUFSIZE) == ENOTENMEM)
        return ENOTENMEM;
    if (http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME) == ENOTENMEM)
        return ENOTENMEM;
    if (H_CLOSE(headers))
        if (http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close") == ENOTENMEM)
            return ENOTENMEM;

    /* if we are going to send a Bad Request, then increase errors counter */
    if (code == BADREQUEST)
        increase_badsyntax();

    if (reason != NULL) {
        /* append a reason phrase. ASSUME MAXBUFFERSIZE IS ENOUGH! */
        snprintf(reasonlen,10,"%d",strlen(reason));
        if (http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),reasonlen) == ENOTENMEM)
            return ENOTENMEM;
        if (http_10_header(resp,CONTENTTYPE,MAXBUFSIZE - strlen(resp),"text/plain") == ENOTENMEM)
            return ENOTENMEM;
        if (crlf(resp,MAXBUFSIZE - strlen(resp)) == ENOTENMEM)
            return ENOTENMEM;

        if (type != HEAD)
            strncat(resp,reason,MAXBUFSIZE - strlen(resp));
        bytes = writen(fd,resp,strlen(resp));
    } else { // append file
        if ((errstats = malloc(sizeof(struct stat))) == NULL)
            return ENOTENMEM;

        if ((filename = malloc((strlen(get_errdocsdir()) + 10)*sizeof(char))) == NULL) {
            free(errstats);
            return ENOTENMEM;
        }
        snprintf(filename,strlen(get_errdocsdir()) + 10,"%s/%d.html",get_errdocsdir(),code);

        assert(stat(filename,errstats) != -1);
        assert(S_ISREG(errstats->st_mode));

        snprintf(reasonlen,10,"%d",(int) errstats->st_size);
        if (http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),reasonlen) == ENOTENMEM) {
            free(errstats);
            free(filename);
            return ENOTENMEM;
        }
        if (http_10_header(resp,CONTENTTYPE,MAXBUFSIZE - strlen(resp),"text/html") == ENOTENMEM) {
            free(errstats);
            free(filename);
            return ENOTENMEM;
        }
        if (crlf(resp,MAXBUFSIZE - strlen(resp)) == ENOTENMEM) {
            free(errstats);
            free(filename);
            return ENOTENMEM;
        }

        if (type != HEAD) {
            file = open(filename,O_RDONLY);
            n = read(file,resp + strlen(resp)*sizeof(char),MAXBUFSIZE - strlen(resp));
            if ((bytes = writen(fd,resp,strlen(resp))) == -1) {
                close(file);
                free(filename);
                free(errstats);
                return bytes;
            }

            if (n == MAXBUFSIZE - strlen(resp))
                while ((n = read(file,resp,MAXBUFSIZE)) > 0)
                    if (bytes += n = writen(fd,resp,strlen(resp))) {
                        bytes = n;
                        break;
                    }

            close(file);
        } else {
            if ((bytes = writen(fd,resp,strlen(resp))) == -1) {
                free(filename);
                free(errstats);
                return bytes;
            }
        }
        free(filename);
        free(errstats);
    }
    
    return bytes;
}

/*
 * Attends stat petitions.
 */
int attend_10_stat_petition(int fd, int headers)
{
    counters_t c;
    char resp[MAXBUFSIZE] = "", buf[MAXBUFSIZE] = "", len[10];

    // write header to a string
    if (http_10_status(resp,OK,MAXBUFSIZE) == ENOTENMEM)
        return ENOTENMEM;
    if (http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME) == ENOTENMEM)
        return ENOTENMEM;
    if (H_CLOSE(headers))
        if (http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close") == ENOTENMEM)
            return ENOTENMEM;
    if (http_10_header(resp,CONTENTTYPE,MAXBUFSIZE - strlen(resp),"text/html") == ENOTENMEM)
        return ENOTENMEM;

    /* compose body */
    snprintf(buf,strlen(STATSHEADER) + 1,"%s",STATSHEADER);
    if ((c = return_counters()) == NULL)
        return ENOTENMEM;

    snprintf(buf + strlen(buf),MAXBUFSIZE - strlen(buf),"%s %ld<br>\n",
             "N&uacute;mero de peticiones locales:",c->local_petitions);
    snprintf(buf + strlen(buf),MAXBUFSIZE - strlen(buf),"%s %ld<br>\n",
             "N&uacute;mero de peticiones remotas:",c->remote_petitions);
    snprintf(buf + strlen(buf),MAXBUFSIZE - strlen(buf),"%s %ld<br>\n",
             "N&uacute;mero de peticiones con sintaxis err&oacute;nea:",c->badsyntax_petitions);
    snprintf(buf + strlen(buf),MAXBUFSIZE - strlen(buf),"%s",STATSFOOTER);

    // content-length header
    snprintf(len,10,"%d",strlen(buf));
    if (http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),len) == ENOTENMEM) {
        free(c);
        return -1;
    }
    if (crlf(resp,MAXBUFSIZE - strlen(resp)) == ENOTENMEM) {
        free(c);
        return -1;
    }
    // write header and body
    if (writen(fd,resp,strlen(resp)) == -1) {
        free(c);
        return -1;
    }
    if (writen(fd,buf,strlen(buf)) == -1) {
        free(c);
        return -1;
    }

    free(c);
    return 0;
}

/*
 * Attends GET petitions.
 */
int attend_10_get_petition(int fd, petition_t petition, int headers)
{
    int file, pathlen, nread, n;
    u_char *host;
    char *filename, value[50] = "";
    char buf[MAXBUFSIZE] = "";
    char resp[MAXBUFSIZE] = "";
    char *contents;
    struct stat *filestats;
    struct tm *t;
    char *strtime;
    DIR *dir;

    /* check if we are asked for stats */
    if (is_stats(petition->request.uri.path)) {
        if (write_log(fd,petition,OK,"requested") == ENOTENMEM)
            return ENOTENMEM;
        return attend_10_stat_petition(fd,headers);
    }

    if ((host = get_host(petition)) == NULL)
        return ENOTENMEM;

    /* compose server path */
    pathlen = strlen(get_webdocsdir()) + strlen(host) + strlen(petition->request.uri.path);
    if ((filename = malloc((pathlen +2)*sizeof(char))) == NULL) {
        free(host);
        return ENOTENMEM;
    }
    /* create path by printing n characters (including \0) */
    snprintf(filename,pathlen +2,"%s/%s%s",get_webdocsdir(),host,petition->request.uri.path);
    free(host);

    /* take a look at path stats */
    if ((filestats = malloc(sizeof(struct stat))) == NULL) {
        free(filename);
        return ENOTENMEM;
    }
    /* stats are ok, file seem to exist */
    if ((t = malloc(sizeof(struct tm))) == NULL) {
        free(filename);
        free(filestats);
        return ENOTENMEM;
    }
    if ((strtime = malloc(26*sizeof(char))) == NULL) {
        free(filename);
        free(filestats);
        free(t);
        return ENOTENMEM;
    }
    
    if (stat(filename,filestats) == -1) {
        if (errno == ENOENT) {
            write_log(fd,petition,NOTFOUND,"default");
            http_10_errmsg(fd,NOTFOUND,NULL,headers,GET);
        } else {
            write_log(fd,petition,FORBIDDEN,strerror(errno));
            http_10_errmsg(fd,FORBIDDEN,strerror(errno),headers,GET);
        }
        free(filename);
        return 0;
    }

    /* save last modification time */
    t = gmtime_r(&filestats->st_mtime,t);
    strtime = asctime_r((const struct tm *) t,strtime);
    strtime[strlen(strtime) - 1] = '\0';
    free(t);

    if (S_ISREG(filestats->st_mode)) {
        /* try to open file */
        file = open(filename,O_RDONLY);
        if (file == -1) {
            free(filestats);
            if (errno == EACCES) {
                write_log(fd,petition,FORBIDDEN,"default");
                http_10_errmsg(fd,FORBIDDEN,NULL,headers,GET);
            } else {
                write_log(fd,petition,INTERROR,strerror(errno));
                http_10_errmsg(fd,INTERROR,strerror(errno),headers,GET);
            }
        } else {
            write_log(fd,petition,OK,"requested");
            n = http_10_status(resp,OK,MAXBUFSIZE);
            sprintf(value,"%d",(int) filestats->st_size);
            n = http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),value);
            n = http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME);
            n = http_10_header(resp,LASTMODIFIED,MAXBUFSIZE - strlen(resp),strtime);
            if (H_CLOSE(headers))
                n = http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close");
            n = crlf(resp,MAXBUFSIZE - strlen(resp));
            if (writen(fd,resp,strlen(resp)) == -1) {
                free(filename);
                free(filestats);
                close(file);
                return -1;
            }
            nread = 0;
            while ((nread = read(file,buf,MAXBUFSIZE)) > 0)
                if (writen(fd,buf,nread) == -1) {
                    free(filename);
                    free(filestats);
                    close(file);
                    return -1;
                }
            close(file);
        }
    } else if (S_ISDIR(filestats->st_mode)) {
        if ((dir = opendir(filename)) == NULL) {
            free(filestats);
            if (errno == EACCES) {
                write_log(fd,petition,FORBIDDEN,"default");
                http_10_errmsg(fd,FORBIDDEN,NULL,headers,GET);
            } else {
                write_log(fd,petition,INTERROR,strerror(errno));
                http_10_errmsg(fd,INTERROR,strerror(errno),headers,GET);
            }
        } else {
            contents = dir2html(dir,petition->request.uri.path,filename);
            write_log(fd,petition,OK,"requested");
            n = http_10_status(resp,OK,MAXBUFSIZE);
            sprintf(value,"%d",strlen(contents));
            n = http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),value);
            n = http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME);
            n = http_10_header(resp,LASTMODIFIED,MAXBUFSIZE - strlen(resp),strtime);
            if (H_CLOSE(headers))
                n = http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close");
            n = crlf(resp,MAXBUFSIZE - strlen(resp));
            if (writen(fd,resp,strlen(resp)) == -1) {
                free(filename);
                free(filestats);
                free(contents);
                closedir(dir);
                return -1;
            }
            if (writen(fd,contents,strlen(contents)) == -1) {
                free(filename);
                free(filestats);
                free(contents);
                closedir(dir);
                return -1;
            }
            free(contents);
        }
        closedir(dir);
    } else {  /* deny  other non-regular files */
        write_log(fd,petition,FORBIDDEN,"default");
        http_10_errmsg(fd,FORBIDDEN,NULL,headers,GET);
    }

    free(strtime);
    free(filename);
    free(filestats);
    return 0;
}

/*
 * Attends HEAD petitions.
 */
int attend_10_head_petition(int fd, struct petition *petition, int headers)
{
    int file, pathlen, n;
    u_char *host;
    char *filename, value[50] = "";
    char resp[MAXBUFSIZE] = "";
    char *contents;
    struct stat *filestats;
    struct tm *t;
    char *strtime;
    DIR *dir;

    if ((host = get_host(petition)) == NULL)
        return ENOTENMEM;

    /* compose server path */
    pathlen = strlen(get_webdocsdir()) + strlen(host) + strlen(petition->request.uri.path);
    if ((filename = malloc((pathlen +2)*sizeof(char))) == NULL) { 
        free(host);
        return ENOTENMEM;
    }
    /* create path by printing n characters (including \0) */
    snprintf(filename,pathlen +2,"%s/%s%s",get_webdocsdir(),host,petition->request.uri.path);
    free(host);

    /* take a look at path stats */
    if ((filestats = malloc(sizeof(struct stat))) == NULL) {
        free(filename);
        return ENOTENMEM;
    }
    /* stats are ok, file seem to exist */
    if ((t = malloc(sizeof(struct tm))) == NULL) {
        free(filename);
        free(filestats);
        return ENOTENMEM;
    }
    if ((strtime = malloc(26*sizeof(char))) == NULL) {
        free(filename);
        free(filestats);
        free(t);
        return ENOTENMEM;
    }

    if (stat(filename,filestats) == -1) {
        if (errno == ENOENT) {
            write_log(fd,petition,NOTFOUND,"default");
            http_10_errmsg(fd,NOTFOUND,NULL,headers,HEAD);
        } else {
            write_log(fd,petition,FORBIDDEN,strerror(errno));
            http_10_errmsg(fd,FORBIDDEN,strerror(errno),headers,HEAD);
        }
        free(filename);
        return 0;
    }

    /* save last modification time */
    t = gmtime_r(&filestats->st_mtime,t);
    strtime = asctime_r((const struct tm *) t,strtime);
    strtime[strlen(strtime) - 1] = '\0';
    free(t);

    if (S_ISREG(filestats->st_mode)) {
        /* try to open file */
        file = open(filename,O_RDONLY);
        if (file == -1) {
            free(filestats);
            if (errno == EACCES) {
                write_log(fd,petition,FORBIDDEN,"default");
                http_10_errmsg(fd,FORBIDDEN,NULL,headers,HEAD);
            } else {
                write_log(fd,petition,INTERROR,strerror(errno));
                http_10_errmsg(fd,INTERROR,strerror(errno),headers,HEAD);
            }
        } else {
            write_log(fd,petition,OK,"requested");
            n = http_10_status(resp,OK,MAXBUFSIZE);
            sprintf(value,"%d",(int) filestats->st_size);
            n = http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),value);
            n = http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME);
            n = http_10_header(resp,LASTMODIFIED,MAXBUFSIZE - strlen(resp),strtime);
            if (H_CLOSE(headers))
                n = http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close");
            n = crlf(resp,MAXBUFSIZE - strlen(resp));
            if (writen(fd,resp,strlen(resp)) == -1) {
                free(filename);
                free(filestats);
                close(file);
                return -1;
            }
            close(file);
        }
    } else if (S_ISDIR(filestats->st_mode)) {
        if ((dir = opendir(filename)) == NULL) {
            free(filestats);
            if (errno == EACCES) {
                write_log(fd,petition,FORBIDDEN,"default");
                http_10_errmsg(fd,FORBIDDEN,NULL,headers,HEAD);
            } else {
                write_log(fd,petition,INTERROR,strerror(errno));
                http_10_errmsg(fd,INTERROR,strerror(errno),headers,HEAD);
            }
        } else {
            contents = dir2html(dir,petition->request.uri.path,filename);
            write_log(fd,petition,OK,"requested");
            n = http_10_status(resp,OK,MAXBUFSIZE);
            sprintf(value,"%d",strlen(contents));
            n = http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),value);
            n = http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME);
            n = http_10_header(resp,LASTMODIFIED,MAXBUFSIZE - strlen(resp),strtime);
            if (H_CLOSE(headers))
                n = http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close");
            n = crlf(resp,MAXBUFSIZE - strlen(resp));
            if (writen(fd,resp,strlen(resp)) == -1) {
                free(filename);
                free(filestats);
                free(contents);
                closedir(dir);
                return -1;
            }
            free(contents);
        }
        closedir(dir);
    } else { /* deny other non-regular files */
        write_log(fd,petition,FORBIDDEN,"default");
        http_10_errmsg(fd,FORBIDDEN,NULL,headers,HEAD);
    }

    free(strtime);
    free(filename);
    free(filestats);
    return 0;
}

/*
 * Attends POST petitions.
 */
int attend_10_post_petition(int fd, struct petition *petition, int headers)
{
    int pathlen, len, n, nwrite, nread, randstatus, randfile, status, file, pos;
    char *filename, tmpfile[50] = "", bodylen[10] = "", tmp[MAXBUFSIZE];
    char buf[MAXBUFSIZE] = "";
    char resp[MAXBUFSIZE] = "";
    struct stat *filestats;
    int srvpipe[2], cgipipe[2];
    pid_t pid;

    status = 0;
    if (pipe(srvpipe) || pipe(cgipipe)) {
        if (write_log(fd,petition,INTERROR,"default") == ENOTENMEM)
            return ENOTENMEM;
        return http_10_errmsg(fd,INTERROR,NULL,headers,POST);
    }

    /* check path */
    if (strlen(petition->request.uri.path) > strlen(get_cgidir()) + 2) {
        if (strncmp(get_cgidir(),petition->request.uri.path + 1*sizeof(char),strlen(get_cgidir())) == 0) {
            // cgi requested
            /* compose server path */
            pathlen = strlen(petition->request.uri.path);    /* include \0 and exclude initial slash */
            if ((filename = malloc((pathlen)*sizeof(char))) == NULL)
                return ENOTENMEM;

            /* create path by printing n characters (including \0) */
            snprintf(filename,pathlen,"%s%s",get_cgidir(),petition->request.uri.path
                + strlen(get_cgidir())*sizeof(char) + 1*sizeof(char));

            /* take a look at path stats */
            if ((filestats = malloc(sizeof(struct stat))) == NULL) {
                free(filename);
                return ENOTENMEM;
            }
            if (stat(filename,filestats) == -1)
                if (errno == ENOENT) {
                    free(filename);
                    if (write_log(fd,petition,NOTFOUND,"default") == ENOTENMEM)
                        return ENOTENMEM;
                    return http_10_errmsg(fd,NOTFOUND,NULL,headers,POST);
                }
        
            if (S_ISREG(filestats->st_mode))
                if (access(filename,X_OK) == 0) {
                    /* get random status exit code */
                    srand((int) time(NULL));
                    randstatus = (100 + random()) % 256;

                    /* calculate body length */
                    if (petition->body == NULL)
                        snprintf(bodylen,2,"0");
                    else
                        snprintf(bodylen,10,"%d",strlen(petition->body));
            
                    if ((pid = fork()) != 0) {    
                    /* server thread */
            
                        close(cgipipe[STDOUT_FILENO]);
                        close(srvpipe[STDIN_FILENO]);

                        n = 0;
                        // write parameters to cgi
                        while ((nwrite = writen(srvpipe[STDOUT_FILENO],petition->body+n*sizeof(char),
                                strlen(petition->body)-n*sizeof(char))) > 0) {
                            if ((nwrite == -1) && (errno != EINTR))
                                break;
                            if (nwrite <= strlen(petition->body + n*sizeof(char)))
                                n = nwrite;
                        }

                        /* prepare temp file where to place cgi contents */
                        do {
                            /* get random temp file number */
                            srand((int) time(NULL));
                            randfile = random();
                            /* generate file name */
                            snprintf(tmpfile,50,"/tmp/%d.%d",pid,randfile);
                            free(filestats);
                            if ((filestats = malloc(sizeof(struct stat))) == NULL) {
                                free(filename);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                return ENOTENMEM;
                            }
                        } while (stat(tmpfile,filestats) != -1);

                        file = open(tmpfile,O_RDWR | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
                        nread = 0;
                        while ((n = read(cgipipe[STDIN_FILENO],buf,MAXBUFSIZE)) != 0) {
                            if ((n == -1) && (errno != EINTR))
                                break;
                            nread += n;
                            if (writen(file,buf,n) == -1) {
                                close(file);
                                remove(tmpfile);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                free(filename);
                                free(filestats);
                                return -1;
                            }
                        }

                        /* if execlp failed, tell the client */
                        wait(&status);
                        if ((n == 0) && (WEXITSTATUS(status) == randstatus)) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return http_10_errmsg(fd,INTERROR,NULL,headers,POST);
                        }

                        /* the cgi executed successfully, so send contents and remove temp file */
                        if (write_log(fd,petition,OK,"requested") == ENOTENMEM) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return ENOTENMEM;
                        }
                        if ((n = http_10_status(resp,OK,MAXBUFSIZE)) == ENOTENMEM) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return ENOTENMEM;
                        }
                        if ((n = http_10_header(resp,SERVER,MAXBUFSIZE - strlen(resp),SERVERNAME)) == ENOTENMEM) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return ENOTENMEM;
                        }
                        if (H_CLOSE(headers))
                            if ((n = http_10_header(resp,CONNECTION,MAXBUFSIZE - strlen(resp),"close")) == ENOTENMEM) {
                                close(file);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                free(filename);
                                free(filestats);
                                remove(tmpfile);
                                return ENOTENMEM;
                            }
                        /* we need to know wether if the CGI added some headers, so search for 2xCRLF */
                        len = get_bodylen(file);
                        snprintf(bodylen,10,"%d",len);
                        if ((n = http_10_header(resp,CONTENTLEN,MAXBUFSIZE - strlen(resp),bodylen)) == ENOTENMEM) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return ENOTENMEM;
                        }
                        if (len == nread) { /* there were no headers in the output of the CGI */
                            if ((n = http_10_header(resp,CONTENTTYPE,MAXBUFSIZE - strlen(resp),"text/plain")) == ENOTENMEM) {
                                close(file);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                free(filename);
                                free(filestats);
                                remove(tmpfile);
                                return ENOTENMEM;
                            }
                            lseek(file,0,SEEK_SET);
                        } else { /* there were some headers, so include them */
                            lseek(file,0,SEEK_SET);
                            n = read(file,buf,((nread - len - 2) < MAXBUFSIZE) ? (nread - len - 2) : MAXBUFSIZE);
                            strncat(resp,buf,((nread - len - 2) < MAXBUFSIZE) ? (nread - len - 2) : MAXBUFSIZE);
                            if ((n = crlf(resp,MAXBUFSIZE - strlen(resp))) == ENOTENMEM) {
                                close(file);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                free(filename);
                                free(filestats);
                                remove(tmpfile);
                                return ENOTENMEM;
                            }
                            lseek(file,(nread - len),SEEK_SET);
                        }
                        /* append double \r\n sequence to headers */
                        if ((n = crlf(resp,MAXBUFSIZE - strlen(resp))) == ENOTENMEM) {
                            close(file);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            remove(tmpfile);
                            return ENOTENMEM;
                        }
                        /* write headers to socket */
                        if (writen(fd,resp,strlen(resp)) == -1) {
                            close(file);
                            remove(tmpfile);
                            close(cgipipe[STDIN_FILENO]);
                            close(srvpipe[STDOUT_FILENO]);
                            free(filename);
                            free(filestats);
                            return -1;
                        }

                        while ((n = read(file,buf,MAXBUFSIZE)) != 0) {
                            if ((n == -1) && (errno != EINTR))
                                break;
                            if (writen(fd,buf,n) == -1) {
                                close(file);
                                remove(tmpfile);
                                close(cgipipe[STDIN_FILENO]);
                                close(srvpipe[STDOUT_FILENO]);
                                free(filename);
                                free(filestats);
                                return -1;
                            }
                            bzero(buf,MAXBUFSIZE);
                        }
                        close(file);
                        remove(tmpfile);

                        close(cgipipe[STDIN_FILENO]);
                        close(srvpipe[STDOUT_FILENO]);
                    } else {    
                    /* forked process */

                        // change working directory
                        if (chdir(get_cgidir()))
                            _exit(randstatus);

                        // set cgi relative name
                        free(filename);
                        pathlen = strlen(petition->request.uri.path) - strlen(get_cgidir()) + 1;
                        if ((filename = malloc(pathlen*sizeof(char))) == NULL)
                            _exit(randstatus);

                        snprintf(filename,pathlen,".%s",petition->request.uri.path +
                                 (strlen(get_cgidir()) + 1)*sizeof(char));

                        /*
                         * Set environment variables before CGI execution.
                         * See draft-coar-cgi-v11-03.txt for more details.
                         */
                        setenv("REQUEST_METHOD","POST",0);
                        setenv("CONTENT_LENGTH",bodylen,0);
                        pos = search_header(petition,PETITION,CONTENTTYPE);
                        if ((pos != -1) && (pos != petition->headerno))
                            setenv("CONTENT_TYPE",petition->header[pos].value,0);
                        setenv("GATEWAY_INTERFACE","CGI/1.1",0);
                        setenv("QUERY_STRING","",0);    /* this is a post query! */
                        setenv("SERVER_SOFTWARE",SERVERNAME,0);
                        pos = search_header(petition,PETITION,AUTHORIZATION);
                        if ((pos != -1) && (pos != petition->headerno))
                            setenv("AUTH_TYPE",petition->header[pos].value,0);
                        setenv("REMOTE_HOST","",0);    /* do not provide this info to CGI */
                        snprintf(tmp,MAXBUFSIZE,"%ld",s_port);
                        setenv("SERVER_PORT",tmp,0);
                        setenv("PATH_INFO",petition->request.uri.path + (strlen(get_cgidir()) + 1)*sizeof(char),0);
                        setenv("SCRIPT_NAME",petition->request.uri.path + (strlen(get_cgidir()) + 2)*sizeof(char),0);
                        setenv("SERVER_PROTOCOL","HTTP/1.0",0);

                        // prepare stdin and out for common gateway interface
                        close(cgipipe[STDIN_FILENO]);
                        close(srvpipe[STDOUT_FILENO]);
                        dup2(cgipipe[STDOUT_FILENO],STDOUT_FILENO);
                        close(cgipipe[STDOUT_FILENO]);
                        dup2(srvpipe[STDIN_FILENO],STDIN_FILENO);
                        close(srvpipe[STDIN_FILENO]);

                        execlp(filename,NULL);

                        /*
                         * if execlp fails, we return here and exit with a random
                         * number so that we can detect non-cgi errors.
                         */
                        fprintf(stderr, "Server error: could not execute CGI.\n");
                        _exit(randstatus);
                    }
                    free(filename);
                    free(filestats);
                    return 0;
                }
            free(filename);
            free(filestats);
        }
    }

    write_log(fd,petition,FORBIDDEN,"default");
    http_10_errmsg(fd,FORBIDDEN,NULL,headers,POST);
    return 0;
} 

int attend_10_petition(const int fd, const petition_t petition, const int headers)
{   
    int pos;
    
    if (!check_host(petition,(petition->request.uri.port_present) ? s_port : 0)) {
        if (write_log(fd,petition,BADREQUEST,"default") == ENOTENMEM)
            return ENOTENMEM;
        return http_10_errmsg(fd,BADREQUEST,NULL,headers,petition->request.method);
    }

    pos = search_header(petition,PETITION,HOST);

    if ((pos != -1) && (pos != petition->headerno))
        if (!is_localdomain(petition->header[pos].value)) {
            if (write_log(fd,petition,BADREQUEST,"default") == ENOTENMEM);
                return ENOTENMEM;
            return http_10_errmsg(fd,BADREQUEST,NULL,headers,petition->request.method);
        }

    increase_local();

    switch (petition->request.method) {
    case GET:
        return attend_10_get_petition(fd,petition,headers);
        break;
    case HEAD:
        return attend_10_head_petition(fd,petition,headers);
        break;
    case POST:
        return attend_10_post_petition(fd,petition,headers);
        break;
    default:
        if (write_log(fd,petition,NOTIMPLEM,"default") == ENOTENMEM)
            return ENOTENMEM;
        http_10_errmsg(fd,NOTIMPLEM,NULL,headers,petition->request.method);
    }
    
    return 0;
}
