/*
 * HTTP 1.0 library definitions
 * File name: http_10.h
 * Author: Jaime Prez Crespo <japecre@lsub.org>
 * Last modification: May 9, 2004
 *
 * Copyright (c) 2004, Jaime Prez Crespo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * - Neither the name of the Rey Juan Carlos University nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __HTTP_10_H
#define __HTTP_10_H

#include "defs.h"
#include "utils.h"
#include "message_analysis.h"

/* unknown status code */
#define UNKNOWN_0	0

#define EBADSTATUS	1

/*
 * Concatenates max n bytes from a request message petition to a string.
 * Returns:
 *   Bytes written at str.
 *   0 elsewhere.
 */
int http_10_request(char *str, const int method, const char *uri, const int n);

/*
 * Concatenates max n bytes from a status message response to a string.
 * Returns:
 *   Bytes written at str.
 *   0 elsewhere.
 */
int http_10_status(char *str, const int status, const int n);

/*
 * Concatenates max n bytes from a HTTP/1.0 header into a string.
 * Returns:
 *   Bytes written at str.
 *   -1 on error.
 */
int http_10_header(char *str, const int header, const int n, const char *value);

/*
 * Writes to a file descriptor a HTTP/1.0 error message corresponding to error code.
 * If reason is not NULL, then it is appended to the message as its body. Else, the
 * function will search at error directory for an appropiate document.
 * It will add headers specified in flags.
 * If type is HEAD, no response body will be written.
 */
ssize_t http_10_errmsg(const int fd,
                       const int code,
                       const char *reason,
                       const int headers,
		       const int type);

/*
 * Attends a petition and writes results into file descriptor supplied.
 * Returns:
 *   -1 if any error.
 * Note:
 *   This function WILL NOT CLOSE the fd descriptor.
 *   It will add headers specified in flags.
 */
int attend_10_petition(const int fd,
                       const petition_t petition,
                       const int headers);

#endif    /* http_10.h */

