/*
 * Server configuration library
 * File name: http_config.h
 * Author: Jaime Prez Crespo <japecre@lsub.org>
 * Last modification: May 10, 2004
 *
 * Copyright (c) 2004, Jaime Prez Crespo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * - Neither the name of the Rey Juan Carlos University nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __SERVERCONFIG_H
#define __SERVERCONFIG_H

#include "defs.h"

/* server wide variables */
u_char *default_domain;
long   s_port;	/* will be defined as a command line argument */
struct in_addr s_addr;
uid_t  uid;

#define MAXBUFSIZE      5120*10
#define SERVERNAME      "RAL/0.5 Server"
#define MAXPATHNAME     PATH_MAX
#define STATSHEADER     "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<html>\n\
<head>\n  <title>Informe de estad&iacute;sticas</title>\n </head>\
<body>\n<h1>Informe de estad&iacute;sticas</h1>\n<p>"
#define STATSFOOTER     "</p>\n<hr/>\n<address>RAL/0.5 Server</address>\n</body>\n</html>"

/* Initializes server configuration */
int init_config(void);

/* Returns the max number of threads */
int get_maxthreads(void);

/* Returns the webdocs directory */
u_char *get_webdocsdir(void);

/* Returns the error docs dir */
u_char *get_errdocsdir(void);

/* Returns the cgi directory */
u_char *get_cgidir(void);

/* Returns the log file name */
u_char *get_logfile(void);

/* Returns the socket timeout */
int get_timeout(void);

/*
 * Sets local domains to those read from config file
 * Returns number of bytes read from config file.
 */
ssize_t set_localdomain(void);

/* Determines if given str represents one of domains served */
int is_localdomain(const u_char *str);

/* Determines if we are asked for server statistics */
int is_stats(const u_char *str);

#endif /* http_config.h */

