/*
 * HTTP server statistics library
 * File name: statistics.h
 * Author: Jaime Prez Crespo <japecre@lsub.org>
 * Last modification: May 3, 2004
 *
 * Copyright (c) 2004, Jaime Prez Crespo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * - Neither the name of the Rey Juan Carlos University nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __STATISTICS_H
#define __STATISTICS_H

#include "defs.h"
#ifndef __MESSAGE_ANALYSIS_H
#include "message_analysis.h"
#endif

#define LOCAL   0
#define REMOTE  1

typedef struct counters *counters_t;

struct counters
{
    u_long local_petitions;
    u_long remote_petitions;
    u_long badsyntax_petitions;
};

/*
 * Statistics management
 */

/*
 * Initializes the server statistics and all internal mechanisms like
 * mutexes for stats managing.
 */
void init_stats(void);

/*
 * Increases by one the local petition counter.
 */
void increase_local(void);

/*
 * Increases by one the remote petition counter.
 */
void increase_remote(void);

/*
 * Increases by one the bad syntax counter.
 */
void increase_badsyntax(void);

/*
 * Returns a counters_t structure with all statistics
 * associated petitions received by server.
 */
counters_t return_counters(void);

/*
 * Resets server statistics.
 */
void reset_stats(void);

/*
 * Log management
 */

/*
 * Initializes server logging mechanisms.
 */
void init_logs(void);

/*
 * Writes a new entry to log file in the form:
 * "CLIENT_IP - [DATE] "REQUESTLINE" CODE "MESSAGE"".
 */
int write_log(const int fd,
              const petition_t petition,
              const int code,
              const char *message);

#endif /* statistics.h */

