/*
 * URI definitions and analysis library
 * File name: uri_analysis.h
 * Author: Jaime Prez Crespo <japecre@lsub.org>
 * Last modification: May 17, 2004
 *
 * Copyright (c) 2004, Jaime Prez Crespo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * - Neither the name of the Rey Juan Carlos University nor the names of its contributors
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
#ifndef __URI_ANALYSIS_H
#define __URI_ANALYSIS_H

#include "defs.h"

/* uri type */
typedef struct uri *uri_t;

/* structure describing all possible fields of an URI */
struct uri
{
    u_char *protocol;    /* http://, optional */
    u_char *host;        /* optional */
    int    port;         /* optional */
    int    port_present; /* mandatory */
    u_char *path;
};

#define URI struct uri	/* define an easy to use struct name */

#define EBADURI 1    /* for bad composed uris */
#define ENALLOC 2    /* if target struct is not allocated */
#define ENOTSEC 3    /* for insecure uris */
#define ENOTMEM 4    /* if not enough memory */

/*
 * Breaks an uri string into a uri structure.
 * Returns:
 *   EBADURI if could not find a valid uri.
 *   ENALLOC if elements was not allocated.
 *   ENOTSEC if uri is found not to be secure.
 *   ENOTMEM if there is not enough memory to allocate data
 *   -1 if any other error occurs.
 */
int break_uri (const u_char *uri, URI *elements);

/*
 * Initializes and returns a new uri structure.
 * NULL if fails.
 */
uri_t init_uri(void);

/*
 * Frees an uri structure.
 */
int free_uri(uri_t *uri);

#endif    /* uri_analysis.h */
 
